//! Util traits to get info from types in [`crate::types`].

use std::fmt::Display;

use crate::types::{Block, ModuleBlockElements, Register, RwSpecifier};

pub(crate) trait GetName {
    fn get_name(&self) -> String;
}

impl GetName for ModuleBlockElements {
    fn get_name(&self) -> String {
        match self {
            ModuleBlockElements::Block(block) => block.get_name(),
            ModuleBlockElements::Register(register) => register.get_name(),
            ModuleBlockElements::Memory(_memory) => todo!(),
            ModuleBlockElements::Fifo(_fifo) => todo!(),
        }
    }
}

impl GetName for Block {
    fn get_name(&self) -> String {
        self.name.clone()
    }
}

impl GetName for Register {
    fn get_name(&self) -> String {
        self.name.clone()
    }
}

impl Display for RwSpecifier {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        let s = match self {
            RwSpecifier::R => "r",
            RwSpecifier::W => "w",
            RwSpecifier::RW => "rw",
        };
        write!(f, "{}", s)
    }
}
